//************************************************************************
// PICIO.cpp
//
// 
//
//
//
// hk@holger-klabunde.de
// www.holger-klabunde.de
// 02.01.2004
// Compiler Visual C++ V5.0
//************************************************************************
#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "hiddll.h"
#include "setupapidll.h"
#include "picio.h"

HANDLE HidHandle;

unsigned char inbuff[USB_BUFFSIZE];   //USB input buffer. 1 byte for report, 8 bytes from the PIC
unsigned char outbuff[USB_BUFFSIZE];  //USB output buffer. 1 byte for report, 8 bytes for the PIC
char msg[1024];

//############################################################
// Reads all five ADC Channels, but returns only the selected
// channel. If you need ADC values closer together in time,
// call this function and read inbuff[2..6] directly.
//. 
unsigned char ReadADC(int adchannel)
//############################################################
{
 int Success;
 unsigned char by;
 unsigned long BytesReturned,BytesWritten;
 DWORD Error;

 by=0;

 outbuff[0]=0;			// report
 outbuff[1]=GET_ADC;	// command for the PIC 
 outbuff[2]=0;			// reserved, make zero
 outbuff[3]=0;			// reserved, make zero
 outbuff[4]=0;			// reserved, make zero
 outbuff[5]=0;			// reserved, make zero
 outbuff[6]=0;			// reserved, make zero
 outbuff[7]=0;			// reserved, make zero
 outbuff[8]=0;			// reserved, make zero
 Success=WriteFile(HidHandle,outbuff,USB_BUFFSIZE,&BytesWritten,NULL);
 if(Success==0)
  {
   Error=GetLastError();
   sprintf(msg,"WriteFile() ErrorCode %d",Error);
   MessageBox(NULL, msg, "Error", MB_OK);
   return 0;
  }

 Success=ReadFile(HidHandle,inbuff,USB_BUFFSIZE,&BytesReturned,NULL);
 if(Success==0)
  {
   Error=GetLastError();
   sprintf(msg,"ReadFile() ErrorCode %d",Error);
   MessageBox(NULL, msg, "Error", MB_OK);
   return 0;
  }

 switch(adchannel)
  {
   case AN0: case AD0: by=inbuff[2]; break;
   case AN1: case AD1: by=inbuff[3]; break;
   case AN2: case AD2: by=inbuff[4]; break;
   case AN3: case AD3: by=inbuff[5]; break;
   case AN4: case AD4: by=inbuff[6]; break;
   default: by=0; break;
  } 

 return by;
}

//############################################################
// Reads all five PORTS, but returns only the selected
// PORT. If you need PORT values closer together in time,
// call this function and read inbuff[2..6] directly.
//. 
unsigned char ReadPicPort(int port)
//#########################################
{
 int Success;
 unsigned char by;
 unsigned long BytesReturned,BytesWritten;
 DWORD Error;

 by=0;

 outbuff[0]=0;			// report
 outbuff[1]=GET_PORTS;	// command for the PIC 
 outbuff[2]=0;			// flag read PORTS 
 outbuff[3]=0;			// reserved, make zero
 outbuff[4]=0;			// reserved, make zero
 outbuff[5]=0;			// reserved, make zero
 outbuff[6]=0;			// reserved, make zero
 outbuff[7]=0;			// reserved, make zero
 outbuff[8]=0;			// reserved, make zero
 Success=WriteFile(HidHandle,outbuff,USB_BUFFSIZE,&BytesWritten,NULL);
 if(Success==0)
  {
   Error=GetLastError();
   sprintf(msg,"WriteFile() ErrorCode %d",Error);
   MessageBox(NULL, msg, "Error", MB_OK);
   return 0;
  }

 Success=ReadFile(HidHandle,inbuff,USB_BUFFSIZE,&BytesReturned,NULL);
 if(Success==0)
  {
   Error=GetLastError();
   sprintf(msg,"ReadFile() ErrorCode %d",Error);
   MessageBox(NULL, msg, "Error", MB_OK);
   return 0;
  }

 switch(port)
  {
   case PORTA: by=inbuff[2]; break;
   case PORTB: by=inbuff[3]; break;
   case PORTC: by=inbuff[4]; break;
   case PORTD: by=inbuff[5]; break;
   case PORTE: by=inbuff[6]; break;
   default: by=0; break;
  } 

 return by;
}

//#############################################
int WritePicPort(int port, unsigned char data)
//#############################################
{
 int Success;
 unsigned long BytesWritten;
 DWORD Error;

 outbuff[0]=0; //Always zero
 switch(port) // insert command code for selected port
  {
   case PORTA: outbuff[1]=SET_PORTA; break;
   case PORTB: outbuff[1]=SET_PORTB; break;
   case PORTC: outbuff[1]=SET_PORTC; break;
   case PORTD: outbuff[1]=SET_PORTD; break;
   case PORTE: outbuff[1]=SET_PORTE; break;
   default: return 0; break;
  } 
 
 outbuff[2]=data;	// portx value
 outbuff[3]=0;		// flag=0 : "set portx not trisx"
 outbuff[4]=0;		// reserved, make zero
 outbuff[5]=0;		// reserved, make zero
 outbuff[6]=0;		// reserved, make zero
 outbuff[7]=0;		// reserved, make zero
 outbuff[8]=0;		// reserved, make zero

 Success=WriteFile(HidHandle,outbuff,USB_BUFFSIZE,&BytesWritten,NULL);
 if(Success==0)
  {
   Error=GetLastError();
   sprintf(msg,"WriteFile() ErrorCode %d",Error);
   MessageBox(NULL, msg, "Error", MB_OK);
   return 0;
  }

 return 1;
}

//##############################################################
// Reads all five TRISx registers, but returns only the selected
// register. If you need values closer together in time,
// call this function and read inbuff[2..6] directly.
//. 
unsigned char ReadTris(int port)
//#########################################
{
 int Success;
 unsigned char by;
 unsigned long BytesReturned,BytesWritten;
 DWORD Error;

 by=0;

 outbuff[0]=0;			// report
 outbuff[1]=GET_PORTS;	// command for the PIC 
 outbuff[2]=1;			// flag read TRIS 
 outbuff[3]=0;			// reserved, make zero
 outbuff[4]=0;			// reserved, make zero
 outbuff[5]=0;			// reserved, make zero
 outbuff[6]=0;			// reserved, make zero
 outbuff[7]=0;			// reserved, make zero
 outbuff[8]=0;			// reserved, make zero
 Success=WriteFile(HidHandle,outbuff,USB_BUFFSIZE,&BytesWritten,NULL);
 if(Success==0)
  {
   Error=GetLastError();
   sprintf(msg,"WriteFile() ErrorCode %d",Error);
   MessageBox(NULL, msg, "Error", MB_OK);
   return 0;
  }

 Success=ReadFile(HidHandle,inbuff,USB_BUFFSIZE,&BytesReturned,NULL);
 if(Success==0)
  {
   Error=GetLastError();
   sprintf(msg,"ReadFile() ErrorCode %d",Error);
   MessageBox(NULL, msg, "Error", MB_OK);
   return 0;
  }

 switch(port)
  {
   case TRISA: case PORTA: by=inbuff[2]; break;
   case TRISB: case PORTB: by=inbuff[3]; break;
   case TRISC: case PORTC: by=inbuff[4]; break;
   case TRISD: case PORTD: by=inbuff[5]; break;
   case TRISE: case PORTE: by=inbuff[6]; break;
   default: by=0; break;
  } 

 return by;
}

//#########################################
int WriteTris(int port, unsigned char data)
//#########################################
{
 int Success;
 unsigned long BytesWritten;
 DWORD Error;

 outbuff[0]=0; //Always zero

 switch(port) // insert command code for selected port
  {
   case PORTA: case TRISA: outbuff[1]=SET_PORTA; break;
   case PORTB: case TRISB: outbuff[1]=SET_PORTB; break;
   case PORTC: case TRISC: outbuff[1]=SET_PORTC; break;
   case PORTD: case TRISD: outbuff[1]=SET_PORTD; break;
   case PORTE: case TRISE: outbuff[1]=SET_PORTE; break;
   default: return 0; break;
  } 
 
 outbuff[2]=0;
 outbuff[3]=1;		// flag=1 : "set trisx not portx"
 outbuff[4]=data;	// tris value
 outbuff[5]=0;		// reserved, make zero
 outbuff[6]=0;		// reserved, make zero
 outbuff[7]=0;		// reserved, make zero
 outbuff[8]=0;		// reserved, make zero

 Success=WriteFile(HidHandle,outbuff,USB_BUFFSIZE,&BytesWritten,NULL);
 if(Success==0)
  {
   Error=GetLastError();
   sprintf(msg,"WriteFile() ErrorCode %d",Error);
   MessageBox(NULL, msg, "Error", MB_OK);
   return 0;
  }

 return 1;
}

//#########################################
// Write a value to the PWM1 Registers
// Maximum resolution is 10 bit (1023).
// Depends on PR2 (PWM frequency)
int WritePWM1(unsigned int pwmvalue)
//#########################################
{
 int Success;
 unsigned long BytesWritten;
 DWORD Error;

 outbuff[0]=0;			//Always zero
 outbuff[1]=SET_PWM;
 outbuff[2]=PWM1;
 outbuff[3]=(unsigned char)(pwmvalue>>2);		 // CCPR1L
 outbuff[4]=(unsigned char)((pwmvalue&0x03)<<4); // CCP1CON 5:4
 outbuff[5]=0;		//
 outbuff[6]=0;		//
 outbuff[7]=0;		//
 outbuff[8]=0;		// reserved, make zero

 Success=WriteFile(HidHandle,outbuff,USB_BUFFSIZE,&BytesWritten,NULL);
 if(Success==0)
  {
   Error=GetLastError();
   sprintf(msg,"WriteFile() ErrorCode %d",Error);
   MessageBox(NULL, msg, "Error", MB_OK);
   return 0;
  }

 return 1;
}

//#########################################
// Write a value to the PWM2 Registers
// Maximum resolution is 10 bit (1023).
// Depends on PR2 (PWM frequency)
int WritePWM2(unsigned int pwmvalue)
//#########################################
{
 int Success;
 unsigned long BytesWritten;
 DWORD Error;

 outbuff[0]=0;			//Always zero
 outbuff[1]=SET_PWM;
 outbuff[2]=PWM2;
 outbuff[3]=0;		//
 outbuff[4]=0;		//
 outbuff[5]=(unsigned char)(pwmvalue>>2);		 // CCPR2L
 outbuff[6]=(unsigned char)((pwmvalue&0x03)<<4); // CCP2CON 5:4
 outbuff[7]=0;		//
 outbuff[8]=0;		// reserved, make zero

 Success=WriteFile(HidHandle,outbuff,USB_BUFFSIZE,&BytesWritten,NULL);
 if(Success==0)
  {
   Error=GetLastError();
   sprintf(msg,"WriteFile() ErrorCode %d",Error);
   MessageBox(NULL, msg, "Error", MB_OK);
   return 0;
  }

 return 1;
}

//#########################################
// Write a value to the PR2 Register
int WritePR2(unsigned char value)
//#########################################
{
 int Success;
 unsigned long BytesWritten;
 DWORD Error;

 outbuff[0]=0;			//Always zero
 outbuff[1]=SET_PWM;
 outbuff[2]=PR2;
 outbuff[3]=0;		//
 outbuff[4]=0;		//
 outbuff[5]=0;
 outbuff[6]=0;
 outbuff[7]=value;	//
 outbuff[8]=0;		// reserved, make zero

 Success=WriteFile(HidHandle,outbuff,USB_BUFFSIZE,&BytesWritten,NULL);
 if(Success==0)
  {
   Error=GetLastError();
   sprintf(msg,"WriteFile() ErrorCode %d",Error);
   MessageBox(NULL, msg, "Error", MB_OK);
   return 0;
  }

 return 1;
}

/*
   outbuff[0]=0;			//report
   outbuff[1]=OUT_SER|(7<<4);	//command for the PIC in low nibble + number of byte in high nibble
   outbuff[2]='H';
   outbuff[3]='e';
   outbuff[4]='l';
   outbuff[5]='l';
   outbuff[6]='o';
   outbuff[7]=0x0d;
   outbuff[8]=0x0a;
   Success=WriteFile(HidHandle,outbuff,USB_BUFFSIZE,&BytesWritten,NULL);
   if(Success==0)
    {
     Error=GetLastError();
     sprintf(zeile,"WriteFile() ErrorCode %d",Error);
     MessageBox(NULL, zeile, "Error", MB_OK);
    }

   MessageBox(NULL, "Tipp bis zu 7 Zeichen ins Terminal", "Hallo", MB_OK);

   outbuff[0]=0;		//report
   outbuff[1]=IN_SER;	//command for the PIC 
   Success=WriteFile(HidHandle,outbuff,USB_BUFFSIZE,&BytesWritten,NULL);
   if(Success==0)
    {
     Error=GetLastError();
     sprintf(zeile,"WriteFile() ErrorCode %d",Error);
     MessageBox(NULL, zeile, "Error", MB_OK);
    }

   Success=ReadFile(HidHandle,inbuff,USB_BUFFSIZE,&BytesReturned,NULL);
   if(Success==0)
    {
     Error=GetLastError();
     sprintf(zeile,"ReadFile() ErrorCode %d",Error);
     MessageBox(NULL, zeile, "Error", MB_OK);
    }
   fprintf(stdout,"%02X %02X %02X %02X %02X %02X %02X %02X %02X\n"
           ,inbuff[0],inbuff[1],inbuff[2],inbuff[3],inbuff[4]
           ,inbuff[5],inbuff[6],inbuff[7],inbuff[8]);

*/
