//************************************************************************
// Setupapi.h
//
// SETUPAPI.DLL function definitions and prototypes declaration
// NOT all functions of SETUPAPI.DLL are defined here !
//
// hk@holger-klabunde.de
// www.holger-klabunde.de
// 26.12.2003
// Compiler Visual C++ V5.0
//************************************************************************


#ifndef _MYSETUPAPI_H
#define _MYSETUPAPI_H

#include <pshpack4.h>

extern void UnloadSetupApiDll();
extern int LoadSetupApiDll();

//
// Define type for reference to device information set
//
typedef PVOID HDEVINFO;

//
// Device information structure (references a device instance
// that is a member of a device information set)
//
typedef struct _SP_DEVINFO_DATA {
    DWORD cbSize;
    GUID  ClassGuid;
    DWORD DevInst;    // DEVINST handle
    DWORD Reserved;
} SP_DEVINFO_DATA, *PSP_DEVINFO_DATA;

//
// Device interface information structure (references a device
// interface that is associated with the device information
// element that owns it).
//
typedef struct _SP_DEVICE_INTERFACE_DATA {
    DWORD cbSize;
    GUID  InterfaceClassGuid;
    DWORD Flags;
    DWORD Reserved;
} SP_DEVICE_INTERFACE_DATA, *PSP_DEVICE_INTERFACE_DATA;

//
// Flags for SP_DEVICE_INTERFACE_DATA.Flags field.
//
#define SPINT_ACTIVE  0x00000001
#define SPINT_DEFAULT 0x00000002
#define SPINT_REMOVED 0x00000004

//
// Backward compatibility--do not use.
//
typedef SP_DEVICE_INTERFACE_DATA  SP_INTERFACE_DEVICE_DATA;
typedef PSP_DEVICE_INTERFACE_DATA PSP_INTERFACE_DEVICE_DATA;
#define SPID_ACTIVE               SPINT_ACTIVE
#define SPID_DEFAULT              SPINT_DEFAULT
#define SPID_REMOVED              SPINT_REMOVED


typedef struct _SP_DEVICE_INTERFACE_DETAIL_DATA_A {
    DWORD  cbSize;
    CHAR   DevicePath[ANYSIZE_ARRAY];
} SP_DEVICE_INTERFACE_DETAIL_DATA_A, *PSP_DEVICE_INTERFACE_DETAIL_DATA_A;

typedef struct _SP_DEVICE_INTERFACE_DETAIL_DATA_W {
    DWORD  cbSize;
    WCHAR  DevicePath[ANYSIZE_ARRAY];
} SP_DEVICE_INTERFACE_DETAIL_DATA_W, *PSP_DEVICE_INTERFACE_DETAIL_DATA_W;

#ifdef UNICODE
typedef SP_DEVICE_INTERFACE_DETAIL_DATA_W SP_DEVICE_INTERFACE_DETAIL_DATA;
typedef PSP_DEVICE_INTERFACE_DETAIL_DATA_W PSP_DEVICE_INTERFACE_DETAIL_DATA;
#else
typedef SP_DEVICE_INTERFACE_DETAIL_DATA_A SP_DEVICE_INTERFACE_DETAIL_DATA;
typedef PSP_DEVICE_INTERFACE_DETAIL_DATA_A PSP_DEVICE_INTERFACE_DETAIL_DATA;
#endif

//
// Backward compatibility--do not use.
//
typedef SP_DEVICE_INTERFACE_DETAIL_DATA_W SP_INTERFACE_DEVICE_DETAIL_DATA_W;
typedef PSP_DEVICE_INTERFACE_DETAIL_DATA_W PSP_INTERFACE_DEVICE_DETAIL_DATA_W;
typedef SP_DEVICE_INTERFACE_DETAIL_DATA_A SP_INTERFACE_DEVICE_DETAIL_DATA_A;
typedef PSP_DEVICE_INTERFACE_DETAIL_DATA_A PSP_INTERFACE_DEVICE_DETAIL_DATA_A;
#ifdef UNICODE
typedef SP_INTERFACE_DEVICE_DETAIL_DATA_W SP_INTERFACE_DEVICE_DETAIL_DATA;
typedef PSP_INTERFACE_DEVICE_DETAIL_DATA_W PSP_INTERFACE_DEVICE_DETAIL_DATA;
#else
typedef SP_INTERFACE_DEVICE_DETAIL_DATA_A SP_INTERFACE_DEVICE_DETAIL_DATA;
typedef PSP_INTERFACE_DEVICE_DETAIL_DATA_A PSP_INTERFACE_DEVICE_DETAIL_DATA;
#endif

//
// Flags controlling what is included in the device information set built
// by SetupDiGetClassDevs
//
#define DIGCF_DEFAULT           0x00000001  // only valid with DIGCF_DEVICEINTERFACE
#define DIGCF_PRESENT           0x00000002
#define DIGCF_ALLCLASSES        0x00000004
#define DIGCF_PROFILE           0x00000008
#define DIGCF_DEVICEINTERFACE   0x00000010

//
// Backward compatibility--do not use.
//
#define DIGCF_INTERFACEDEVICE DIGCF_DEVICEINTERFACE


typedef HDEVINFO (WINAPI* DLL_SetupDiGetClassDevsA)(
    LPGUID ClassGuid,
    PCSTR  Enumerator,
    HWND   hwndParent,
    DWORD  Flags
    );
extern DLL_SetupDiGetClassDevsA SetupDiGetClassDevsA;

typedef HDEVINFO (WINAPI* DLL_SetupDiGetClassDevsW)(
    LPGUID ClassGuid,
    PCWSTR Enumerator,
    HWND   hwndParent,
    DWORD  Flags
    );
extern DLL_SetupDiGetClassDevsW SetupDiGetClassDevsW;

#ifdef UNICODE
#define DLL_SetupDiGetClassDevs DLL_SetupDiGetClassDevsW
#define SetupDiGetClassDevs SetupDiGetClassDevsW
#else
#define DLL_SetupDiGetClassDevs DLL_SetupDiGetClassDevsA
#define SetupDiGetClassDevs SetupDiGetClassDevsA
#endif

typedef BOOL (WINAPI* DLL_SetupDiEnumDeviceInterfaces)(
    HDEVINFO                  DeviceInfoSet,
    PSP_DEVINFO_DATA          DeviceInfoData,
    LPGUID                    InterfaceClassGuid,
    DWORD                     MemberIndex,
    PSP_DEVICE_INTERFACE_DATA DeviceInterfaceData
    );
extern DLL_SetupDiEnumDeviceInterfaces SetupDiEnumDeviceInterfaces;

typedef BOOL (WINAPI* DLL_SetupDiGetDeviceInterfaceDetailA)(
    HDEVINFO                           DeviceInfoSet,
    PSP_DEVICE_INTERFACE_DATA          DeviceInterfaceData,
    PSP_DEVICE_INTERFACE_DETAIL_DATA_A DeviceInterfaceDetailData,
    DWORD                              DeviceInterfaceDetailDataSize,
    PDWORD                             RequiredSize,
    PSP_DEVINFO_DATA                   DeviceInfoData
    );
extern DLL_SetupDiGetDeviceInterfaceDetailA SetupDiGetDeviceInterfaceDetailA;

typedef BOOL (WINAPI* DLL_SetupDiGetDeviceInterfaceDetailW)(
    HDEVINFO                           DeviceInfoSet,
    PSP_DEVICE_INTERFACE_DATA          DeviceInterfaceData,
    PSP_DEVICE_INTERFACE_DETAIL_DATA_W DeviceInterfaceDetailData,
    DWORD                              DeviceInterfaceDetailDataSize,
    PDWORD                             RequiredSize,
    PSP_DEVINFO_DATA                   DeviceInfoData
    );
extern DLL_SetupDiGetDeviceInterfaceDetailW SetupDiGetDeviceInterfaceDetailW;

#ifdef UNICODE
#define DLL_SetupDiGetDeviceInterfaceDetail DLL_SetupDiGetDeviceInterfaceDetailW
#define SetupDiGetDeviceInterfaceDetail SetupDiGetDeviceInterfaceDetailW
#else
#define DLL_SetupDiGetDeviceInterfaceDetail DLL_SetupDiGetDeviceInterfaceDetailA
#define SetupDiGetDeviceInterfaceDetail SetupDiGetDeviceInterfaceDetailA
#endif


typedef BOOL (WINAPI* DLL_SetupDiDestroyDeviceInfoList)(
    HDEVINFO DeviceInfoSet
    );
extern DLL_SetupDiDestroyDeviceInfoList SetupDiDestroyDeviceInfoList;


#include <poppack.h>

#endif //_MYSETUPAPI_H
