; ###############################################################################
;	filename:		USB_MAIN.ASM
;				Sample mainline program
;###############################################################################
;
;    	Author:			Holger Klabunde
;    	Company:		
;
;	Date:				03.01.2004
;	Assembled using		MPASM 3.50
;	Configuration Bits:	H4 Oscillator, WDT Off, Power up timer off
;
;################################################################################
;
;	include files:
;	P16C765.inc		Rev 1.00
;	usb_defs.inc	Rev 1.10
;
;################################################################################
#include <p16c765.inc>
#include "usb_defs.inc"
	errorlevel -302 ; supress "register not in bank0, check page bits" message

	__CONFIG	_H4_OSC & _WDT_OFF & _PWRTE_OFF & _CP_OFF
	__IDLOCS 	H'1000'	; My Softwareversion for ID Locations

unbanked	udata_shr
W_save		res	1

bank0	udata
Status_save	res	1	; registers for saving context
PCLATH_save	res	1
FSR_save	res	1

inner		res	1	; delay loop counters
middle		res	1
outer		res	1

outbuffer	res	8	; buffer for data received from host
inbuffer	res	8	; buffer for data sent to host

rx_buffer	res	7	; UART Receive Buffer
rx_count	res	1	; UART RX byte count down.
rx_full		res	1	; UART buffer is full

	extern	InitUSB
	extern	PutEP1
	extern	GetEP1
	extern	ServiceUSBInt
	extern	CheckSleep
	extern	RemoteWakeup

STARTUP	code
 	pagesel	main
	goto	main
	nop
InterruptServiceVector
	movwf	W_save		; save W
	movf	STATUS,W
	clrf	STATUS		; force to page 0
	movwf	Status_save	; save STATUS
	movf	PCLATH,w
	movwf	PCLATH_save	; save PCLATH
	movf	FSR,w
	movwf	FSR_save	; save FSR

; *************************************************************
; Interrupt Service Routine
; First we step through several stages, attempting to identify the source 
; of the interrupt.
; ******************************************************************

Process_ISR
; Step 1, what triggered the interrupt?

	btfsc	INTCON,T0IF	; Timer 0
	nop

	btfsc	INTCON,RBIF	; Port B
	nop

	btfsc	INTCON,INTF	; External Interrupt
	nop

	BANKSEL	PIR1
	pagesel	ServiceUSBInt
	btfsc	PIR1,USBIF
	call	ServiceUSBInt


	BANKSEL	PIR1
	pagesel	USART_rx
	btfsc	PIR1,RCIF
	call	USART_rx

	btfsc	PIR1,ADIF	; AD Done?
	nop

	btfsc	PIR1,TXIF
    nop	

	btfsc	PIR1,CCP1IF
	nop

	btfsc	PIR1,TMR2IF
	nop
	
	btfsc	PIR1,TMR1IF
	nop

	btfsc	PIR2,CCP2IF
	nop

; ******************************************************************
; End ISR, restore context and return to the main program
; ******************************************************************
EndISR
	clrf	STATUS		; select bank 0
	movf	FSR_save,w	; restore the FSR
	movwf	FSR
	movf	PCLATH_save,w	; restore PCLATH
	movwf	PCLATH
	movf	Status_save,w	; restore Status
	movwf	STATUS
	swapf	W_save,f	; restore W without corrupting STATUS
	swapf	W_save,w
	retfie

	code
; **********************************************************************
; Receiver Interrupt Handler.  Add byte to the receive queue
;  if more than 7 bytes are received before the buffer is emptied..
;  the newest data is dropped.
; **********************************************************************
USART_rx
	BANKSEL	rx_full
	btfss	rx_full,0
	goto	USART_receive	; buffer is not full.. get the byte
	BANKSEL	RCREG
	movf	RCREG,w		; buffer is full.. unload receiver and drop it
	goto	USART_rx_done
USART_receive
	BANKSEL	rx_buffer
	movlw	rx_buffer
	addwf	rx_count,w
	BANKSEL	FSR
	movwf	FSR		; setup the FSR to receive data
	BANKSEL	RCREG
	movf	RCREG,w		; get the data
	BANKSEL	INDF
	movwf	INDF		; store the data

	BANKSEL	rx_count
	incf rx_count,f
	movf rx_count,w
	sublw 7				;rx_count = 7 ? then queue is full 
    btfss STATUS,Z
	goto	USART_rx_done	; if the buffer has room. we are done
	bsf	rx_full,0	;  otherwise, rx_full is set
USART_rx_done
	return

; ******************************************************************
; test program that sets up the buffers and calls the ISR for processing.
; 	
; ******************************************************************
main
	movlw	.60		; delay 16 uS to wait for USB to reset
	movwf	W_save		; SIE before initializing registers
	decfsz	W_save,f	; W_save is merely a convienient register
	goto	$-1		; to use for the delay counter.

	BANKSEL	OPTION_REG
	bsf	OPTION_REG,NOT_RBPU	; turn off pullups

	BANKSEL	TRISB			;PORTB all outputs (default)
	clrf	TRISB
	BANKSEL	PORTB
	clrf	PORTB

	BANKSEL	TRISC			;PORTC RC0=TMR1 input, RC1=PWM1, RC2=PWM2, RC6=TxD, RC7=RxD
    movlw	b'10111001'
	movwf	TRISC
	BANKSEL	PORTC
	clrf	PORTC
	BANKSEL	PR2			
    movlw	d'255'
	movwf	PR2
	BANKSEL	CCPR1L			
	clrf	CCPR1L			; Duty Cycle 0%
	BANKSEL	CCPR2L			
	clrf	CCPR2L			; Duty Cycle 0%
	BANKSEL	CCP1CON			
    movlw	b'00001100'		;PWM Mode
	movwf	CCP1CON
	BANKSEL	CCP2CON			
    movlw	b'00001100'		;PWM Mode
	movwf	CCP2CON
	BANKSEL	T2CON			
    movlw	b'00000100'		;Prescaler 1:1, TMR2 on
	movwf	T2CON

    ;PORTD, E all inputs. You can change this later via USB	 
    
	pagesel	InitUSB
	call	InitUSB			; start the USB system

	ConfiguredUSB			; wait here until the enumeration process is complete

	BANKSEL	TRISA			; Initialize the ADC converter
	movlw	0x3F			; Five analog inputs on PORTA, PORTA4 also input
	movwf	TRISA
	BANKSEL	ADCON1
	movlw	0x02			; Five analog inputs on PORTA
	movwf	ADCON1			; Vref is VDD

	; configure the serial port
	; 9600 Baud 8N1 asynchron, high speed
	
	BANKSEL	SPBRG
	;	movlw	38		;38400 Baud
	;	movlw	77		;19200 Baud
	movlw 	d'155'		;9600 Baud
	movwf	SPBRG

	BANKSEL	TXSTA
	bsf	TXSTA, BRGH			;BRGH=1

	BANKSEL	RCSTA
	bsf	RCSTA, SPEN	;SPEN=1

	banksel	PIE1	
	bsf	PIE1, RCIE		; enable USART interrupt

	BANKSEL	rx_full
	clrf	rx_full
	BANKSEL	rx_count
	clrf	rx_count

	clrf	outbuffer
	clrf	outbuffer+1
	clrf	outbuffer+2
	clrf	outbuffer+3
	clrf	outbuffer+4
	clrf	outbuffer+5
	clrf	outbuffer+6
	clrf	outbuffer+7

	clrf	inbuffer
	clrf	inbuffer+1
	clrf	inbuffer+2
	clrf	inbuffer+3
	clrf	inbuffer+4
	clrf	inbuffer+5
	clrf	inbuffer+6
	clrf	inbuffer+7

	BANKSEL	TXSTA
	bsf	TXSTA,TXEN      ;TXEN=1
	BANKSEL	RCSTA
	bsf	RCSTA,CREN	;CREN=1

	nop
; Begin main loop
LoopForData
	
	pagesel	GetEP1
	banksel outbuffer
	bcf	STATUS,IRP
	movlw	outbuffer
	movwf	FSR
	movlw	0x8
	call	GetEP1

	pagesel	ProcessOUTBuffer
	btfsc	STATUS,C	; If we got a packet
	call	ProcessOUTBuffer ; process it

; Don't do this. Send data only if requested
;	pagesel	ProcessINBuffer
;	call	ProcessINBuffer ; make a new packet

;	pagesel	PutEP1
;	bcf	STATUS,IRP
;	banksel inbuffer
;	movlw	inbuffer
;	movwf	FSR
;	movlw	0x8
;	call	PutEP1

; OERR vom Receive UART lschen falls gesetzt
	pagesel	LoopForData
	BANKSEL	RCSTA
	btfss	RCSTA,OERR
	goto	LoopForData	; Try again
    bcf     RCSTA,CREN  ; Reset receive logic
    nop
    bsf     RCSTA,CREN  ; Reset receive logic
	goto	LoopForData	; Try again
; End main loop

;##############################################################
; Sends 9 bytes: one zero + 8 bytes inbuffer
;##############################################################
SendINBuffer
	pagesel	PutEP1
	bcf	STATUS,IRP
	banksel inbuffer
	movlw	inbuffer
	movwf	FSR
	movlw	0x8
	call	PutEP1
	return

;#####################################################################
; Sends up to 7 bytes from the uart receive queue
; inbuff+0 high nibble is number of bytes (0..7) from rx_count
; low nibble is flag 0x03 for uart data
; inbuff+1 is the first byte from the receive queue
; if rx_count is less then 7 ignore all data above. its old garbage.
;#####################################################################
UartRXData
    BANKSEL rx_count 
	movf	rx_count,w	; compare to the rx counter...
	iorlw	0x30		; rx_count is low nibble, flag is high nibble
    BANKSEL inbuffer 
	movwf	inbuffer+0
	swapf	inbuffer+0,f ; rx_count now is high nibble, flag is low nibble
	movf	rx_buffer,w  ; rx_buffer+0 was received first
	movwf	inbuffer+1
	movf	rx_buffer+1,w
	movwf	inbuffer+2
	movf	rx_buffer+2,w
	movwf	inbuffer+3
	movf	rx_buffer+3,w
	movwf	inbuffer+4
	movf	rx_buffer+4,w
	movwf	inbuffer+5
	movf	rx_buffer+5,w
	movwf	inbuffer+6
	movf	rx_buffer+6,w
	movwf	inbuffer+7

	clrf	rx_count	; reset serial receive counter
	clrf	rx_full		; activate receive queue
    pagesel SendINBuffer
    call SendINBuffer
    return

;###########################################################
; Pass five ADC values
;###########################################################
SendADC
	movlw	2			;flag ADC data
	BANKSEL inbuffer
	movwf	inbuffer
	movlw	0
	PAGESEL	ReadADC
	call	ReadADC
	BANKSEL	inbuffer+1
	movwf	inbuffer+1
	movlw	1
	PAGESEL	ReadADC
	call	ReadADC
	BANKSEL	inbuffer+2
	movwf	inbuffer+2
	movlw	2
	PAGESEL	ReadADC
	call	ReadADC
	BANKSEL	inbuffer+3
	movwf	inbuffer+3
	movlw	3
	PAGESEL	ReadADC
	call	ReadADC
	BANKSEL	inbuffer+4
	movwf	inbuffer+4
	movlw	4
	PAGESEL	ReadADC
	call	ReadADC
	BANKSEL	inbuffer+5
	movwf	inbuffer+5
    pagesel SendINBuffer
    call SendINBuffer
	return

;###########################################################
; Process the data
; outbuffer+0 is the command
;###########################################################
ProcessOUTBuffer
	movlw	high	add_location
	BANKSEL	PCLATH
	movwf	PCLATH
	movlw	low	add_location
	BANKSEL	inner
	movwf	inner
	BANKSEL	outbuffer
	movf	outbuffer,w
	andlw	0x0F		; limit our commands to 16
	BANKSEL	inner
	addwf	inner,f
	BANKSEL	STATUS
	btfsc	STATUS,C
	incf	PCLATH,f
add_location
	addwf	PCL,f
	goto	do_nothing	; command 0x00
	goto	set_pwm		; command 0x01
	goto	get_adc		; command 0x02
	goto	get_serial	; command 0x03
	goto	send_serial	; command 0x04 writes the following bytes to the serial port.
						; high nibble is the actual number of bytes to send. (1-7)
	goto	set_porta	; command 0x05 byte 1 is sent to PORTA or TRISA
	goto	set_portb	; command 0x06 byte 1 is sent to PORTB or TRISB
	goto	set_portc	; command 0x07 byte 1 is sent to PORTC or TRISC
	goto	set_portd	; command 0x08 byte 1 is sent to PORTD or TRISD
	goto	set_porte	; command 0x09 byte 1 is sent to PORTE or TRISE
	goto	get_ports	; command 0x0A read PORTA to PORTE or TRISA to TRISE 
	goto	do_nothing	; command 0x0B
	goto	do_nothing	; command 0x0C
	goto	do_nothing	; command 0x0D
	goto	do_nothing	; command 0x0E
	goto	do_nothing	; command 0x0F just padding out the jump table to 16 entries				
;#####################################################
set_pwm
    pagesel SetPWM
	call SetPWM
	return
;#####################################################
get_adc
    pagesel SendADC
	call SendADC
	return
;#####################################################
get_serial
    pagesel UartRXData
	call UartRXData
	return
;#####################################################
send_serial
	; byte count is in the high nibble
	swapf	outbuffer,f
	movf	outbuffer,w
	andlw	0x0F
	movwf	inner
	; byte count is now inner
	bankisel outbuffer+1
	bcf	STATUS,IRP
	movlw	outbuffer+1
	movwf	FSR
serial_loop	
	movf	INDF,w
	PAGESEL tx_data
	call	tx_data
	incf	FSR,f
	decfsz	inner,f
	goto	serial_loop
	return
;#######################################################
; If somebody wants to switch PORTA to digital IO we
; have to disable the ADC. This is NOT done here yet !
; Then we also need a new function to reenable the ADC
; if needed.
;#######################################################
set_porta
    banksel outbuffer
    btfsc 	outbuffer+2,0 	;flag set tris and not port ?
    goto 	set_trisa
	movf	outbuffer+1,w
	banksel	PORTA
	movwf	PORTA
	return
set_trisa
	movf	outbuffer+3,w
	banksel	TRISA
	movwf	TRISA
	return
;#####################################################
set_portb
    banksel outbuffer
    btfsc 	outbuffer+2,0 	;flag set tris and not port ?
    goto 	set_trisb
	movf	outbuffer+1,w
	banksel	PORTB
	movwf	PORTB
	return
set_trisb
	movf	outbuffer+3,w
	banksel	TRISB
	movwf	TRISB
	return
;#####################################################
set_portc
    banksel outbuffer
    btfsc 	outbuffer+2,0 	;flag set tris and not port ?
    goto 	set_trisc
	movf	outbuffer+1,w
	banksel	PORTC
	movwf	PORTC
	return
set_trisc
	movf	outbuffer+3,w
	banksel	TRISC
	movwf	TRISC
	return
;#####################################################
set_portd
    banksel outbuffer
    btfsc 	outbuffer+2,0 	;flag set tris and not port ?
    goto 	set_trisd
	movf	outbuffer+1,w
	banksel	PORTD
	movwf	PORTD
	return
set_trisd
	movf	outbuffer+3,w
	banksel	TRISD
	movwf	TRISD
	return
;#####################################################
set_porte
    banksel outbuffer
    btfsc 	outbuffer+2,0 	;flag set tris and not port ?
    goto 	set_trise
	movf	outbuffer+1,w
	banksel	PORTE
	movwf	PORTE
	return
set_trise
	movf	outbuffer+3,w
	banksel	TRISE
	movwf	TRISE
	return
;#####################################################
get_ports
	movlw	0x0A		; flag get_ports
	BANKSEL inbuffer
	movwf	inbuffer
    banksel outbuffer
    btfsc 	outbuffer+1,0 	;flag get tris and not port ?
    goto 	get_tris
	banksel	PORTA
	movf	PORTA,w
    andlw	b'00111111'	; only 6 pins
	BANKSEL inbuffer+1
	movwf	inbuffer+1
	banksel	PORTB
	movf	PORTB,w
	BANKSEL inbuffer+2
	movwf	inbuffer+2
	banksel	PORTC
	movf	PORTC,w
    andlw	b'11000111'	; only 5 pins
	BANKSEL inbuffer+3
	movwf	inbuffer+3
	banksel	PORTD
	movf	PORTD,w
	BANKSEL inbuffer+4
	movwf	inbuffer+4
	banksel	PORTE
	movf	PORTE,w
    andlw	b'00000111'	; only 3 pins
	BANKSEL inbuffer+5
	movwf	inbuffer+5
    pagesel SendINBuffer
    call SendINBuffer
    return
;#####################################################
get_tris
	banksel	TRISA
	movf	TRISA,w
    andlw	b'00111111'	; only 6 pins
	BANKSEL inbuffer+1
	movwf	inbuffer+1
	banksel	TRISB
	movf	TRISB,w
	BANKSEL inbuffer+2
	movwf	inbuffer+2
	banksel	TRISC
	movf	TRISC,w
    andlw	b'11000111'	; only 5 pins
	BANKSEL inbuffer+3
	movwf	inbuffer+3
	banksel	TRISD
	movf	TRISD,w
	BANKSEL inbuffer+4
	movwf	inbuffer+4
	banksel	TRISE
	movf	TRISE,w
    andlw	b'00000111'	; only 3 pins
	BANKSEL inbuffer+5
	movwf	inbuffer+5
    pagesel SendINBuffer
    call SendINBuffer
    return
do_nothing
done
	return

;###################################################################
; transmit byte in w
;###################################################################
tx_data
	BANKSEL	PIR1
	btfss	PIR1,TXIF	; wait for TX to complete
	goto	$-1
	BANKSEL	TXREG
	movwf	TXREG		; send the character
	return

;###########################################################
; configure the channel in W and return a conversion in W
;###########################################################
ReadADC
	BANKSEL	inner
	movwf	inner
	rlf	inner,f
	rlf	inner,f
	rlf	inner,w
	andlw	0x38
	iorlw	b'11000001'
	BANKSEL	ADCON0
	movwf	ADCON0
	movlw	.20		; delay 11 uS to acquire the signal
	movwf	inner		; 
	decfsz	inner,f		; 
	goto	$-1		; 

	bsf	ADCON0,GO	; start the conversion
	btfsc	ADCON0,GO
	goto	$-1		; loop until done
	BANKSEL	ADRES
	movf	ADRES,W		; move result into W
	return

;###########################################################
; configure the PWM Module
;###########################################################
SetPWM
    banksel outbuffer
    btfsc 	outbuffer+1,0 	; set PWM1 ?
    call 	SetPWM1
    banksel outbuffer
    btfsc 	outbuffer+1,1 	; set PWM2 ?
    call 	SetPWM2
    banksel outbuffer
    btfsc 	outbuffer+1,2 	; set PR2 ?
    call 	SetPR2
	return
;###########################################################
SetPWM1
    banksel CCP1CON			;change LSB first
    movf	CCP1CON,W
	andlw	b'11001111'		;set bits 5:4 to zero
    banksel outbuffer+3
    addwf	outbuffer+3,W
    banksel CCP1CON
	movwf	CCP1CON
    banksel outbuffer+2
	movf	outbuffer+2,W	
    banksel CCPR1L			;change MSB
	movwf	CCPR1L
	return
;###########################################################
SetPWM2
    banksel CCP2CON			;change LSB first
    movf	CCP2CON,W
	andlw	b'11001111'		;set bits 5:4 to zero
    banksel outbuffer+5
    addwf	outbuffer+5,W
    banksel CCP2CON
	movwf	CCP2CON
    banksel outbuffer+4
	movf	outbuffer+4,W	
    banksel CCPR2L			;change MSB
	movwf	CCPR2L
	return
;###########################################################
SetPR2
    banksel outbuffer+6
	movf	outbuffer+6,W	
    banksel PR2
	movwf	PR2
	return

; this is the
	end

